--***********************************************************************
--									*
--									*
--   This software was written by Bevin Brett, of Digital Equipment	*
--   Corporation.							*
--									*
--   Digital assumes no responsibility AT ALL for the use or reliability*
--   of this software.							*
--									*
--   Redistribution and use in source and binary forms are permitted	*
--   provided that the above copyright notice and this paragraph are	*
--   duplicated in all such forms and that any documentation,		*
--   advertising materials, and other materials related to such		*
--   distribution and use acknowledge that the software was developed	*
--   by Digital Equipment Corporation. The name of Digital Equipment	*
--   Corporation may not be used to endorse or promote products derived	*
--   from this software without specific prior written permission.	*
--									*
--   THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR	*
--   IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED	*
--   WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.*
--									*
--***********************************************************************

with CALENDAR, MATH_LIB;
pragma ELABORATE(CALENDAR);

package body SYSTEM_INTERFACE is

    function DIGITS_SQRT(A : SOME_FLOAT) return SOME_FLOAT is
    package SOME_FLOAT_MATH_LIB is new MATH_LIB(SOME_FLOAT);
    begin
	return SOME_FLOAT_MATH_LIB.SQRT(A);
    end;

    function GET_CPU_TIME return DURATION is
    begin
	return 0.01;
    end;

    function RANDOM return FLOAT is
	function RTL_RANDOM return INTEGER;
	    pragma INTERFACE(C, RTL_RANDOM);                                
	    pragma IMPORT_FUNCTION(RTL_RANDOM, external=>"rand");   --VMS
    begin
	return FLOAT(RTL_RANDOM)/FLOAT(INTEGER'last);
    end;


    function SIN(X : FAST_FLOAT) return FAST_FLOAT is
    package FAST_FLOAT_MATH_LIB is new MATH_LIB(FAST_FLOAT);
    begin
	return FAST_FLOAT_MATH_LIB.SIN(X);
    end;

    function ASIN(X : FAST_FLOAT) return FAST_FLOAT is
    package FAST_FLOAT_MATH_LIB is new MATH_LIB(FAST_FLOAT);
    begin
	return FAST_FLOAT_MATH_LIB.ASIN(X);
    end;

    function ALLOCATE_MEMORY(NUMBER_OF_BYTES : POSITIVE) return ADDRESS is
	function MALLOC(NUMBER_OF_BYTES : POSITIVE) return ADDRESS;
	    pragma INTERFACE(C, MALLOC);
	    pragma IMPORT_FUNCTION(MALLOC, external=>"malloc",
		mechanism=>(VALUE),
		result_mechanism=>VALUE);
    begin
	return MALLOC(NUMBER_OF_BYTES);
    end;

begin
    declare
	-- Start the random number generator "at random"
	--
	procedure SRANDOM(SEED : INTEGER);
	    pragma INTERFACE(C, SRANDOM);
	    pragma IMPORT_PROCEDURE(SRANDOM, external=>"srand", mechanism => value);	--VMS
	function GETPID return INTEGER;
	    pragma INTERFACE(C, GETPID);
	    pragma IMPORT_FUNCTION(GETPID, external=>"getpid");
    begin
	SRANDOM(GETPID);
    end;
end;
